/* includes */

#include <limits>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <iostream>
#include <math.h>
#include <assert.h>
#include "main.h"
#include "dataset.h"
#include "common.h"

using namespace std;

/* globals */
OPT o;



/* prototypes */
int main(int argc, char **argv);
void initOpt(int argc, char **argv);
void help();
void cleanUp();

/* extern */


/*******************************************/

#define STDDEV_STEP 0.2
#define STDDEV_BEGIN 0.0
#define STDDEV_END 2.0
#define STDDEV_STEPS (int)(((STDDEV_END-STDDEV_BEGIN)/STDDEV_STEP)+1)



int main(int argc, char **argv) {

    cout.precision(4);
    cerr.precision(4);
    
    initOpt(argc, argv);

    DataSet dataSet;


    if(o.synthetic) {
        assert("we do not use synthetic data" && 0);
        dataSet.randomWalks(o.syntheticN, o.syntheticLength);
    }    else {
        dataSet.readFromFile(o.rfile);
    }

    char s[1024];


    /*
    // mixed normal: 0.4 1.0
    dataSet.perturbateNormalMixed(0.4, 1.0);
    sprintf(s, "%s_dust_normal_mixed_.dat", o.wfile);
    dataSet.writeDust(s);
    sprintf(s, "%s_mnch_normal_mixed_.dat", o.wfile);
    dataSet.writeMunich(s);
    */

    // mixed exponential: 0.4 1.0
    dataSet.perturbateExponentialMixed(0.0, 1.0);
    sprintf(s, "%s_dust_exponential_mixed_.dat", o.wfile);
    dataSet.writeDust(s);
    //sprintf(s, "%s_mnch_wrongnormal_mixed_.dat", o.wfile);
    //dataSet.writeMunich(s);


    /*dataSet.perturbateUNEMixed(0.4, 1.0);
    sprintf(s, "%s_dust_une_mixed_.dat", o.wfile);
    dataSet.writeDust(s);
    sprintf(s, "%s_mnch_une_mixed_.dat", o.wfile);
    dataSet.writeMunich(s);*/

 return 0;

    if(1)
        for(double stddev = STDDEV_BEGIN; stddev < STDDEV_END+STDDEV_STEP/2.0; stddev += STDDEV_STEP) { 

        // uniform
        dataSet.perturbateUniform(stddev);
        sprintf(s, "%s_dust_uniform_%.1f_.dat", o.wfile, stddev);
        dataSet.writeDust(s);
        sprintf(s, "%s_mnch_uniform_%.1f_.dat", o.wfile, stddev);
        dataSet.writeMunich(s);

        // normal
        dataSet.perturbateNormal(stddev);
        sprintf(s, "%s_dust_normal_%.1f_.dat", o.wfile, stddev);
        dataSet.writeDust(s);
        sprintf(s, "%s_mnch_normal_%.1f_.dat", o.wfile, stddev);
        dataSet.writeMunich(s);

        // exponential
        dataSet.perturbateExponential(stddev);
        sprintf(s, "%s_dust_exponential_%.1f_.dat", o.wfile, stddev);
        dataSet.writeDust(s);
        sprintf(s, "%s_mnch_exponential_%.1f_.dat", o.wfile, stddev);
        dataSet.writeMunich(s);

    }
    cleanUp();
}



void  initOpt(int argc, char **argv) {
    int            c;

    o.synthetic = false;
    o.syntheticLength = -1;
    o.rfile = NULL;
    o.wfile = NULL;

    if (argc ==1)
        help();

    opterr = 0;

    while ((c = getopt(argc, argv, "r:s:N:w:W:")) != EOF)
        switch (c)
        {

        case 's':
        o.syntheticLength = atoi(optarg);
        o.synthetic = true;
        break;

        case 'N':
        o.syntheticN = atoi(optarg);
        o.synthetic = true;
        break;

        case 'r':
        SAFE_FREE(o.rfile);
        o.rfile = strdup(optarg);
        o.synthetic = false;
        break;

        case 'w':
        SAFE_FREE(o.wfile);
        o.wfile = strdup(optarg);
        break;

        default:
        FATAL("option '" << (char) optopt << "' invalid");
    }

    if(o.synthetic) {
        if(o.syntheticLength ==-1 || o.syntheticN == -1) {
            FATAL("synthetic dataset generation requires length,n parameters");
        }

        if(o.syntheticN > TIMESERIES_MAX) {
            FATAL("TIMESERIES_MAX lower than the synthetic timeseries you are asking for");
        }

    } else {
        if(!o.rfile)  {
            FATAL("real dataset generation requires pathname of raw data");
        }
    }


}

void help() {

}

void cleanUp() {
    SAFE_FREE(o.rfile);
    SAFE_FREE(o.wfile);
}
