/* includes */

#include <limits>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <iostream>

#include "main.h"
#include "dataset.h"
#include "common.h"
#include "dust.h"
#include "timeseries.h"
#include "timeseriescollection.h"
#include "euclidean.h"
#include "precisionrecallM.h"
#include <boost/random.hpp>
using namespace boost;


using namespace std;

/* globals */
OPT o;

#define LOOKUPTABLES_PATHNAME "lookuptablesMixed"


const char *names[] = {"50words",
                       "Adiac",
                       "Beef",
                       "CBF",
                       "Coffee",
                       "ECG200",
                       "FaceAll",
                       "FaceFour",
                       "FISH",
                       "Gun_Point",
                       "Lighting2",
                       "Lighting7",
                       "OliveOil",
                       "OSULeaf",
                       "SwedishLeaf",
                       "synthetic_control",
                       "Trace",
                       NULL};

/*

const char *names[] = {"50words_t60x6",
                       "Adiac_t60x6",
                       "Beef_t60x6",
                       "CBF_t60x6",
                       "Coffee_t60x6",
                       "ECG200_t60x6",
                       "FaceAll_t60x6",
                       "FaceFour_t60x6",
                       "FISH_t60x6",
                       "Gun_Point_t60x6",
                       "Lighting2_t60x6",
                       "Lighting7_t60x6",
                       "OliveOil_t60x6",
                       "OSULeaf_t60x6",
                       "SwedishLeaf_t60x6",
                       "synthetic_control_t60x6",
                       "Trace_t60x6",
                       NULL};
*/

bool active[] = {
    false, //    "50words",
    false, //   "Adiac",
    false, //   "Beef",
    false, //   "CBF",
    true, //   "Coffee",
    false, //   "ECG200",
    false, //   "FaceAll",
    false, //   "FaceFour",
    false, //   "FISH",
    false, //   "Gun_Point",
    false, //    "Lighting2",
    false, //    "Lighting7",
    false, //   "OliveOil",
    false, //  "OSULeaf",
    false, //   "SwedishLeaf",
    false,  //   "synthetic_control",
    false //  "Trace", // 17 datasets
};

/* prototypes */
int main(int argc, char **argv);
void initOpt(int argc, char **argv);
void help();
void cleanUp();
void experiment1(TimeSeriesCollection &db, double threshold);
void experiment0(TimeSeriesCollection &db);
void experiment2();
void experiment3(TimeSeriesCollection &db);
void experiment4();
void experiment5();
void experiment6();
void printResultSet(char *name, vector<int> &v);
void printPrecisionRecall(char *name, PrecisionRecallM &pr);
void generateFDustTables();

/* extern */


/*******************************************/


int main(int argc, char **argv) {
    initOpt(argc, argv);

    cout.precision(4);
    cerr.precision(4);


    if(0) {
        experiment2();
    }

    if(1) {
        //   generateFDustTables();
        //  experiment4();
        experiment6();
    }

    if(0) {
        TimeSeriesCollection db;
        DUST dust(db);

        for(double distance = 0.0; distance <= 10.0; distance += 0.1) {
            RandomVariable x = RandomVariable(RANDVAR_NORMAL, 0, 0,0.2);
            RandomVariable y = RandomVariable(RANDVAR_NORMAL, distance, distance,0.8);
            double d = dust.dust(x, y);
            cerr << "distribution:" << RANDVAR_NORMAL << " stddevs:" << 0.2 << " "<< 0.8  << " distance:" << distance << " dustdistance:" << d << endl;
        }
    }


    if(0) {
        TimeSeriesCollection db;
        DUST dust(db);

        double distribution = RANDVAR_UNIFORM;

        for(double stddev = 0.2; stddev <= 0.8; stddev += 0.2) {
            for(double distance = 0.0; distance <= 10.0; distance += 0.1) {
                RandomVariable x = RandomVariable(distribution, 0, 0,stddev);
                RandomVariable y = RandomVariable(distribution, distance, distance,stddev);
                double d = dust.dust(x, y);
                cerr << "distribution:" << distribution << " stddev:" << stddev << " distance:" << distance << " dustdistance:" << d << endl;

            }
        }
    }

    /*

http://www.aquaphoenix.com/lecture/matlab10/page2.html
http://mathworld.wolfram.com/NormalDistribution.html
http://en.wikipedia.org/wiki/Normal_distribution
http://reference.wolfram.com/mathematica/ref/NormalDistribution.html
http://www.mathworks.com/help/toolbox/stats/normpdf.html
http://easycalculation.com/statistics/learn-normal-pdf.php
http://www.boost.org/doc/libs/1_44_0/libs/math/doc/sf_and_dist/html/math_toolkit/dist/stat_tut/overview/generic.html

*/


    cleanUp();
}


void experiment4_evaluate(TimeSeriesCollection &db, DUST &dust, vector<PairIdDistance> &topIdDists, PrecisionRecallM &pr, PrecisionRecallM &prEuclidean, PrecisionRecallM &prRand) {
    Euclidean euclideanTrue(db, true);
    Euclidean euclidean(db);
    

    /*
    vector<int> fiveRandomIds;
    fiveRandomIds.push_back(56);
    fiveRandomIds.push_back(12);
    fiveRandomIds.push_back(40);
    fiveRandomIds.push_back(21);
    fiveRandomIds.push_back(30);

    for(unsigned int i = 0; i < 5; i++) {
        TimeSeries &query = db.sequences[fiveRandomIds[i]-1];
        */

    for(unsigned int i = 0; i < db.sequences.size(); i++) {
        TimeSeries &query = db.sequences[i];

        //

        double threshold = topIdDists[query.id-1].second;

        vector<int> exact = euclideanTrue.rangeQuery(query, threshold);
        vector<int> r;

        double dustThreshold = dust.distance(query, db.sequences[topIdDists[query.id-1].first-1]);
        double euclideanThreshold =euclidean.distance(query, db.sequences[topIdDists[query.id-1].first-1]);

        pr.addStartTime();
       r = dust.rangeQuery(query, dustThreshold);
        pr.addStopTime();
        pr.add(exact, r);
        assert(!isnan(pr.getF1()));

        prEuclidean.addStartTime();
        r = euclidean.rangeQuery(query, euclideanThreshold);
        prEuclidean.addStopTime();
        prEuclidean.add(exact, r);

        int size = r.size(); // used by prRand




        // now, we put |r| random ids in prRand:
        {
            r.clear();
            mt19937 rng(static_cast<unsigned> (time(0)));
            uniform_int<> dist(0, db.sequences.size()-1);
            variate_generator<boost::mt19937&, boost::uniform_int<> > drawSample(rng, dist);
            //cout << "using random ids: ";
            for(int i = 0; i < size; i++) {
                int id = drawSample()+1;
                //cout << id << " ";
                r.push_back(id);
            }
            //cout << "\n";
            prRand.add(exact, r);
            prRand.addStartTime();
            prRand.addStopTime();
        }

        assert(!isnan(prEuclidean.getF1()));

    }
}


void experiment5() {
    char inputPathname[1024];


    for(int i = 0; names[i]; i++) {
        sprintf(inputPathname, "udatasets/%s/%s_dust_%s_%.1f_.dat", names[i], names[i], "uniform", 0.2);
        TimeSeriesCollection db(inputPathname, 1);
        Euclidean euclideanTrue(db, true);
        double desiredRatio = 0.1;
        double threshold = euclideanTrue.getHeuristicThreshold(desiredRatio);
        cout << "dataset: " << names[i] << " desiredRatio:" << desiredRatio << " => threshold: " << threshold << endl;
    }


}


void experiment4() {
    char inputPathname[1024];
    char outputPathname[1024];
    const char *dustlookupTables = LOOKUPTABLES_PATHNAME;



    const char *distribs[] = { "uniform", "normal", "exponential", NULL};

    //double desiredRatio = 0.1;

    for(int i = 0; names[i]; i++) {
     //   if(!active[i])
       //     continue;

        vector<PairIdDistance> topIdDists;
        bool initializeIdDists = true;


        for(int j = 0; distribs[j]; j++) {
          if(j != 1) // XXX just normal is enabled.
          continue;

            sprintf(outputPathname, "results/%s_dust_%s.dat", names[i], distribs[j]);

            cout << "output file: " << outputPathname << "\n";
            ofstream fout(outputPathname);
            if(!fout) {
                FATAL("error opening outputfile");
            }
            fout.precision(4);

            for(double stddev = STDDEV_BEGIN; stddev <= STDDEV_END+STDDEV_STEP/2.0; stddev +=STDDEV_STEP) {

                cout << "processing dataset:" << names[i] << " distrib:" << distribs[j] << " stddev:" << stddev << endl;
                sprintf(inputPathname, "udatasets/%s/%s_dust_%s_%.1f_.dat", names[i], names[i], distribs[j], stddev);

                TimeSeriesCollection db(inputPathname, j+1);

                cout << "input file: " << inputPathname << "\n";

                if(initializeIdDists) {
                    db.computeTopKthresholds(10, topIdDists);
                    initializeIdDists = false;
                }

                DUST dust(db, dustlookupTables);
                PrecisionRecallM pr("DUST");
                PrecisionRecallM prEuclidean("Euclidean");
                PrecisionRecallM prRand("RANDOM");
                PrecisionRecallM prMoving("Moving");


                experiment4_evaluate(db, dust, topIdDists , pr, prEuclidean, prRand);
                fout << j << " " << stddev << " " << pr.getPrecision() <<  " " << pr.getRecall() << " " << pr.getF1() << " " << pr.getTime() << " " <<
                        prEuclidean.getPrecision() << " " << prEuclidean.getRecall() << " " <<
                        prEuclidean.getF1() << " "  << prEuclidean.getTime() << " " <<
                        prRand.getPrecision() << " " << prRand.getRecall() << " " << prRand.getF1() << " " <<
                        prMoving.getPrecision() << " " << prMoving.getRecall() << " " <<
                        prMoving.getF1() << " " << prMoving.getTime() <<  endl;

                fout.flush();
            }

            fout.close();

        }
    }
}



void experiment6() {
    char inputPathname[1024];
    char outputPathname[1024];
    const char *dustlookupTables = LOOKUPTABLES_PATHNAME;

    for(int i = 0; names[i]; i++) {
     if(!active[i])
      continue;


        vector<PairIdDistance> topIdDists;
        bool initializeIdDists = true;


        sprintf(outputPathname, "results/%s_dust_exponential_mixed.dat", names[i]);

        cout << "output file: " << outputPathname << "\n";
        ofstream fout(outputPathname);
        if(!fout) {
            FATAL("error opening outputfile");
        }
        fout.precision(4);

        cout << "processing dataset:" << names[i] << " distrib: .. stddev: mixed\n";
        sprintf(inputPathname, "udatasets/%s/%s_dust_exponential_mixed_.dat", names[i], names[i]);

        TimeSeriesCollection db(inputPathname, 2); // XXX fixing distribution to normal.

        cout << "input file: " << inputPathname << "\n";



        if(initializeIdDists) {
            db.computeTopKthresholds(10, topIdDists);
            initializeIdDists = false;
        }

        DUST dust(db, dustlookupTables);
        PrecisionRecallM pr("DUST");
        PrecisionRecallM prEuclidean("Euclidean");
        PrecisionRecallM prRand("RANDOM");

        Euclidean euclidean(db);

        experiment4_evaluate(db, dust, topIdDists , pr, prEuclidean, prRand);
        fout << -1 << " " << "-1" << " " << pr.getPrecision() <<  " " << pr.getRecall() << " " << pr.getF1() << " " << pr.getTime() << " " <<
                prEuclidean.getPrecision() << " " << prEuclidean.getRecall() << " " << prEuclidean.getF1() << " "  << prEuclidean.getTime() <<
                " " << prRand.getPrecision() << " " << prRand.getRecall() << " " << prRand.getF1() << " " << prRand.getTime() << " " <<            endl;


        prEuclidean.print();
        pr.print();

        fout.flush();
        fout.close();

    }
}


void experiment3(TimeSeriesCollection &db) {
    DUST dust(db);
    Euclidean euclidean(db);

    for(unsigned int i = 0; i < db.sequences[0].sequence.size(); i++) {
        double d1 = euclidean.distance(db.sequences[0], db.sequences[200], i+1);
        double d2=  dust.distance(db.sequences[0], db.sequences[200], i+1);
        cout << "distance: " << i+1 << " " << d1 << " " << d2 << "\n";
    }



}

void generateFDustTables() {
    TimeSeriesCollection db;
    DUST dust(db);
    dust.buildFDustTables(o.wfile);
}


void experiment2() {
    TimeSeriesCollection db(o.rfileCollection, 2);
    Euclidean euclideanTrue(db, true);
    Euclidean euclidean(db);

    double normalization = db.sequences[0].sequence.size();

    for(unsigned int i = 0; i < db.sequences.size(); i++) {
        for(unsigned int j = i+1; j < db.sequences.size(); j++) {
            TimeSeries &t1 = db.sequences[i];
            TimeSeries &t2 = db.sequences[j];
            cout << "normalizedDistance(" << t1.id << "," << t2.id << "): "
                    "EuclTrue=" << euclideanTrue.distance(t1,t2)/normalization  << " " <<
                    "Eucl=" << euclidean.distance(t1, t2)/normalization << " " << endl;
        }
    }


}

void experiment0(TimeSeriesCollection &db) {
    Euclidean euclideanTrue(db, true);

    if(0) {
        for(unsigned int i = 0; i < db.sequences.size(); i++) {
            for(unsigned int j = i+1; j < db.sequences.size(); j++) {
                TimeSeries &t1 = db.sequences[i];
                TimeSeries &t2 = db.sequences[j];
                cout << "euclNormTrue distance(" << t1.id << "," << t2.id << ")=" << euclideanTrue.distance(t1, t2) << endl;
            }
        }
    }

    double desiredRatio = 0.10;
    double threshold = euclideanTrue.getHeuristicThreshold(desiredRatio);
    cout << "threshold returning " << desiredRatio << " of the dataset: " << threshold << "\n";


}



void experiment1(TimeSeriesCollection &db, double threshold) {

    Euclidean euclideanTrue(db, true);
    Euclidean euclidean(db);
    DUST dust(db, "ltnormal");


    PrecisionRecallM prEuclidean("Euclidean");
    PrecisionRecallM prDust("DUST");

    cout << "experiment: performing " << db.sequences.size() << " range queries, averaging results on " << 1  << " runs.\n";

    for(unsigned int i = 0; i < db.sequences.size(); i++) {
        TimeSeries &query = db.sequences[i];

        cout << "[" << i+1 <<  "/" << db.sequences.size() << "]\n";

        vector<int> exact = euclideanTrue.rangeQuery(query, threshold);
        vector<int> r;

        //        cerr << "largest: " << euclideanTrue.largestDistanceId << endl;
        double dustThreshold = dust.distance(query, db.sequences[euclideanTrue.largestDistanceId-1]);
        double euclideanThreshold = euclidean.distance(query, db.sequences[euclideanTrue.largestDistanceId-1]);

        //      cerr << "threshold:" << threshold << " dustThreshold:" << dustThreshold << " euclideanThreshold:" << euclideanThreshold << endl;

        r= euclidean.rangeQuery(query, euclideanThreshold);
        prEuclidean.add(exact, r);

        r = dust.rangeQuery(query, dustThreshold);
        prDust.add(exact, r);

    }


    prEuclidean.print();
    prDust.print();


}

void  initOpt(int argc, char **argv) {
    int            c;

    o.synthetic = false;
    o.syntheticLength = -1;
    o.rfileCollection = NULL;
    o.rfileQuery = NULL;
    o.rfileQuery2 = NULL;
    o.wfile = NULL;

    if (argc ==1)
        help();

    opterr = 0;

    while ((c = getopt(argc, argv, "C:Q:q:w:")) != EOF)
        switch (c)
        {


        case 'C':
        SAFE_FREE(o.rfileCollection);
        o.rfileCollection = strdup(optarg);
        break;

        case 'q':
        SAFE_FREE(o.rfileQuery);
        o.rfileQuery = strdup(optarg);
        break;

        case 'Q':
        SAFE_FREE(o.rfileQuery2);
        o.rfileQuery2 = strdup(optarg);
        break;

        case 'w':
        SAFE_FREE(o.wfile);
        o.wfile = strdup(optarg);
        break;

        default:
        FATAL("option '" << (char) optopt << "' invalid");
    }



}

void help() {

}

void cleanUp() {
    SAFE_FREE(o.rfileCollection);
    SAFE_FREE(o.rfileQuery);
    SAFE_FREE(o.rfileQuery2);
    SAFE_FREE(o.wfile);
}

