#include "timeseriescollection.h"
#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <assert.h>
#include <algorithm>
#include "euclidean.h"
#include "common.h"

using namespace std;

TimeSeriesCollection::TimeSeriesCollection()
{
}

TimeSeriesCollection::TimeSeriesCollection(const char *pathname, int distribution, int limitN) {
    readFromFile(pathname, distribution, limitN);
}

void TimeSeriesCollection::readFromFile(const char *pathname, int distribution, int limitN) {
    ifstream fin(pathname);

    if(fin == NULL) {
        FATAL("unable to read timeseriescollection from file");
    }

    //cerr << "Reading timeseries collection from file " << pathname << "...\n";

    string line;

    while(getline(fin, line)) {
        sequences.push_back(TimeSeries(line, distribution));
        sequences[sequences.size()-1].id = sequences.size();
        if(limitN != -1 && (int)sequences.size() == limitN)
            break;
    }

    cerr << "Read " << sequences.size() << " timeseries from " << pathname<< ", length=" << sequences[0].sequence.size() << endl;

    for(unsigned int i =0; i < sequences.size(); i++ ) {


    }

    // cerr << "Done.\n";

    fin.close();
}


struct mycompare {
    bool operator()(PairIdDistance const &a, PairIdDistance const &b) {
        return a.second < b.second;
    }
};


void TimeSeriesCollection::computeTopKthresholds(unsigned int K, vector<PairIdDistance> &topIdDists) {

    assert(sequences.size() >= K);
    topIdDists.resize(sequences.size());

    Euclidean trueEuclidean(*this, true);

    for(unsigned int i = 0; i < sequences.size(); i++) {
        TimeSeries &q = sequences[i];
        vector<PairIdDistance> distances;

        for(unsigned int j = 0; j < sequences.size(); j++) {
            TimeSeries &t = sequences[j];
            PairIdDistance d =PairIdDistance(t.id, trueEuclidean.distance(q, t));
            distances.push_back(d);
        }

        std::sort(distances.begin() , distances.end(), mycompare());
        topIdDists[q.id-1] = (distances[K]); // K and not K-1 because we have q in the resultset! so we use the K+1th, offset K.
    }
}
