#include <stdlib.h>
#include <iostream>
#include <assert.h>
#include "movingeuclidean.h"
#include <math.h>
#include "common.h"
#include <errno.h>
#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <fstream>
#include <string.h>

#define SMOOTHING_FACTOR 5

MovingEuclidean::MovingEuclidean(TimeSeriesCollection &collection, int variant)
{
    this->variant = variant;

    this->collection = collection;


    // default values:
    w = 5;
    l = 1;


}



vector<int> MovingEuclidean::rangeQuery(TimeSeries &ts, double threshold) {
    vector<int> ids;

    for(unsigned int i = 0; i < collection.sequences.size(); i++) {
        if(distance(ts, collection.sequences[i]) <= threshold) {
            ids.push_back(i+1);
        }
    }

    return ids;
}

double MovingEuclidean::movingAverage(TimeSeries &ts, int pos, int n) {
    double mean = 0;
    double count = 0;

    for(int i = pos-n; i <= pos+n; i++) {
        if(i < 0 || i >= (int)ts.sequence.size())
            continue;


        RandomVariable randvar = ts.sequence[i];

        double decayingfactor = -1;
        double probabfactor = -1;


        switch(variant) {
        case VARIANT_MOVINGAVG:
            decayingfactor = 1.0;
            probabfactor = 1.0;
            break;

        case VARIANT_EXPMOVINGAVG:
            decayingfactor = exp(-ABS(pos-i));
            probabfactor = 1.0;
            break;

        case VARIANT_P_MOVINGAVG:
            decayingfactor = 1.0;
            probabfactor = 1.0 / (randvar.stddev);
            break;

        case VARIANT_P_EXPMOVINGAVG:

            decayingfactor = exp(-l*ABS(pos-i));  //exp(-ABS(pos-i));
            probabfactor = 1.0 / (randvar.stddev);
            break;

        default:
            assert(0 && "WTF!?");
        }


        if(randvar.stddev == 0)
            probabfactor = 1;

        double weight = decayingfactor *  probabfactor ; 

        mean += randvar.observation * weight;

        count+= weight;
    }

    mean /= count;

    assert(count > 0);
    assert(!isnan(mean));

    return mean;
}

double MovingEuclidean::distance(TimeSeries &ts1, TimeSeries &ts2) {
    if(ts1.sequence.size() != ts2.sequence.size()) {
        FATAL("ts1 length=" << ts1.sequence.size() << " != ts2 length=" << ts2.sequence.size());
    }

    double dist = 0;

    for(unsigned int i = 0; i < ts1.sequence.size(); i++ ) {
        dist += pow(movingAverage(ts1,i,w)-movingAverage(ts2,i,w), 2);
    }

    dist = sqrt(dist);

    return dist;
}

void MovingEuclidean::writeToFile(int id, const char *pathname) {

    ofstream fout(pathname);

    if(!fout) {
        FATAL("error opening outputfile " << pathname << ": " << strerror(errno));
    }


    fout.precision(4);

    TimeSeries ts = collection.sequences[id-1];

    for(unsigned int i = 0; i < ts.sequence.size(); i++) {
        fout << movingAverage(ts, i, SMOOTHING_FACTOR) << "\n";
    }

    fout.close();
}
