#include <assert.h>
#include <math.h>
#include <sys/time.h>
#include "precisionrecallM.h"

PrecisionRecall::PrecisionRecall(const char *s) {
    n = 0;
    precision = 0;
    recall = 0;
    size = 0;
    this->s = s;
    t = 0;
    tn = 0;
}


void PrecisionRecall::addStartTime() {
    gettimeofday(&begin, NULL);
    tn++;
}

void PrecisionRecall::addStopTime() {

    struct timeval end, elapsed;

    gettimeofday(&end, NULL);

    TIMEVAL_SUB(&end, &begin, &elapsed);
    t += elapsed.tv_sec * 1000000 + elapsed.tv_usec;

}

void PrecisionRecall::add(vector<int> &exact, vector<int> &estimate) {

    n++;

    if(estimate.size() ==0) {
        if(exact.size() == 0) {
            precision +=1;
            recall += 1;
        }
        return;
    }

    double count = 0;
    for(unsigned int i = 0; i < exact.size(); i++) {
        for(unsigned int j = 0; j < estimate.size(); j++) {
            if(exact[i] == estimate[j]) {
                count++;
                break;
            }
        }
    }

    size += exact.size();

    if(exact.size() == 0) {
        if(estimate.size() == 0) {
            precision += 1;
            recall +=1;
        }
    } else {
        if(estimate.size() > 0) {
            precision += count / ((double)estimate.size());
            recall += count / ((double)exact.size());
        }

        assert(!isnan(precision));
        assert(!isnan(recall));
    }
}

double PrecisionRecall::getPrecision() {
    return precision / n;
}

double PrecisionRecall::getRecall() {
    return recall / n;
}

double PrecisionRecall::getF1() {
    if(precision == 0 && recall == 0) {
        return 0;
    }

    double p = precision / n;
    double r = recall / n;
    return 2*(p*r)/(p+r);
}

void PrecisionRecall::print() {
    cout << s << " (precision recall F1 n) = " << getPrecision() << " " << getRecall() << " " << getF1() << " " << (size/n) << endl;
}

double PrecisionRecall::getTime() {
    return t / tn;
}
