/*
 * Approximator.h
 *
 *  Created on: Apr 12, 2011
 *      Author: besa
 */

#ifndef APPROXIMATOR_H_
#define APPROXIMATOR_H_

#include <vector>
#include <map>
#include <string>
#include <sys/types.h>
#include "DataReader.h"
using namespace std;

class Approximator {
public:
	typedef struct Interval{
		double min;
		double max;
		long double noSamples;
	};
	typedef std::pair< std::pair<int, double> , int> MaxValues;
	typedef std::pair<Interval, MaxValues > IntervalAndMax;

	typedef std::vector<Interval> ApproximatedTimeSlotType;
	typedef std::vector<ApproximatedTimeSlotType> ApproximatedSeriesType;
	typedef std::map<string, ApproximatedSeriesType> ApproximatedSeriesMapType;
	//timeseries name, lower bound probability, upper bound probability
	typedef std::pair<string, Interval> tuple;
	typedef std::pair<string, std::pair<int, int> > tuple_next;
	ApproximatedSeriesMapType approximated_series;
	ApproximatedSeriesType approximated_query;
	DataReader reader;
	vector<tuple> qref;
	vector<tuple_next> qref_next;
	double epsilon;
	double tao;
	int noSamples;
	Approximator();
	Approximator(double e);
	virtual ~Approximator();
	void approximate(int level);
	void approximate_distances_level1();
	void approximate_distances_level2();
	class Search_predicate {
		public:
			string name;
			bool operator() (const tuple& el)
			{
				return el.first == name;
			}
		};
	class Search_predicate_next {
		public:
			string name;
			bool operator() (const tuple_next& el)
			{
				return el.first == name;
			}
		};
private:
	void approximate_level1();
	void approximate_level2();
	IntervalAndMax calculate_distance_interval(ApproximatedSeriesType ats1,
			ApproximatedSeriesType ats2, int selected_c1[], int selected_c2[]);
	Interval calculate_probability_interval(ApproximatedSeriesType ats1, ApproximatedSeriesType ats2, string ts_name);
	bool getNextCombination(int current[], int size, int final_combination[]);
	long double get_total_distances(ApproximatedSeriesType ats1, ApproximatedSeriesType ats2);
	long double get_total_distances_intervals(ApproximatedSeriesType ats1, ApproximatedSeriesType ats2);
	struct sort_pred_upper_bound {
		bool operator() (const tuple& left, const tuple& right)
		{
			return left.second.max > right.second.max;
		}
	};


};

#endif /* APPROXIMATOR_H_ */
