/*
 * DataReader.cpp
 *
 *  Created on: Apr 8, 2011
 *      Author: besa
 */

#include "DataReader.h"
#include "Benchmark.h"
#include <string>
#include <iostream>
#include <fstream>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/random/uniform_int.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/random.hpp>
using namespace std;
using namespace boost;
boost::mt19937 gen;

DataReader::DataReader(){

}

DataReader::DataReader(string file, int collectionLength, int tsLength) {
	DATA_PATH = file;
	this->COLLECTION_SIZE = collectionLength;
	this->LENGTH = tsLength;
}

DataReader::~DataReader() {
	// TODO Auto-generated destructor stub
}

void DataReader::read(int noSamples)
{
	Benchmark benchmark("READING DATA FILES");
	benchmark.start(true);
	string filename = DATA_PATH;
	ifstream seriesfile (filename.c_str());
	string line;
	UncertainSeriesType us;
	int counter = 0;
	int lengthRead = 0;
	if (seriesfile.is_open())
	{
		while ( seriesfile.good() )
		{
			  getline (seriesfile,line);
			  if (line.length() == 0)
				  continue;
			  if (line == "--")
			  {
				  counter++;
				  if (counter <= COLLECTION_SIZE)
					  this->series.insert(std::pair<string, UncertainSeriesType>("timeseries" + boost::lexical_cast<std::string>(counter - 1), us));
				  us.clear();
				  UncertainSeriesType us;
				  lengthRead = 0;
				  continue;
			  }
			  if (lengthRead >= LENGTH)
				  continue;
			  TimeSlotType values;
			  vector<string> tmp;
			  boost::split(tmp, line, boost::is_any_of(":"));
			  string values_line = tmp[1];
			  values_line = values_line.substr(1);
			  vector<string> samples;
			  boost::split(samples, values_line, boost::is_any_of(" "));
			  int drawnSamples = 0;
			  string drawnIndeces = "";
			  vector<int> processed;
			  uniform_int<> ur(0, samples.size() - 1);
			  variate_generator<mt19937&,uniform_int<> > drawSample(gen, ur);
			  while (drawnSamples < noSamples)
			  {
				  int drawnIndex = drawSample();
				  int exists = std::find(processed.begin(), processed.end(), drawnIndex)!=processed.end();
				  if (exists == 1)
				  {
					  continue;
				  }
				  processed.push_back(drawnIndex);
				  try
				  {
					  double d = lexical_cast<double>(samples[drawnIndex]);
					  values.push_back(d);
				  }
				  catch( boost::bad_lexical_cast const& )
				  {
					  cout << samples[drawnIndex] << "casting data to double error" << endl;
					  cout << values_line;
				  }
				  drawnSamples++;
			  }
			  processed.clear();
			  us.push_back(values);
			  lengthRead++;
		}
		seriesfile.close();
	}
	else
	{
		cout << "Unable to open input file " << endl;
		exit(1);
	}
	std::cout << "Total number of series read: " << this->series.size() << endl;
	string filename_query = QUERY_PATH;
	benchmark.end(true);
}

void DataReader::print()
{
	Benchmark benchmark("PRINTING DATA");
	benchmark.start(true);
	SeriesMapType::iterator p;
	for(p = this->series.begin(); p != this->series.end(); p++)
	{
		cout << p->first << endl;
		UncertainSeriesType series = p->second;
		for (unsigned int i = 0; i < series.size(); i++)
		{
			string line = "";
			TimeSlotType samples = series[i];
			for (unsigned int j = 0; j < samples.size(); j++)
			{
				cout << samples[j] << '\t';
			}
			cout << endl;
		}
	}
	benchmark.end(true);
}

