/*
 * TestQueryEngine.cpp
 *
 *  Created on: Apr 15, 2011
 *      Author: besa
 */
#include "DataReader.h"
#include "Approximator.h"
#include "QueryEngine.h"
#include "Benchmark.h"
#include "GroundTruthCalculator.h"
#include "Test.h";
#include <boost/lexical_cast.hpp>
#include <fstream>
#include <boost/random/uniform_int.hpp>
#include <boost/random.hpp>
#include <boost/algorithm/string.hpp>


using namespace boost;
using namespace std;
boost::mt19937 gen2;


void run_fixed_variance();
void run_fixed_tao();
vector< vector<int> >  read_ground_truth(string folder, string dataset, double eps_threshold);
double calculate_threshold(int topk, int query_i, string folder, string dataset);
//int main();

const unsigned int COLLECTION_SIZE = 60;
const unsigned int LENGTH = 6;
const unsigned int NO_QUERIES = 5;
const unsigned int TOPK = 10;
const unsigned int NOSAMPLES = 5;

int main()
{
	string input;
	int answer = 0;
	cout << "Which experiment do you want to run?" << endl;
	cout << "Press 1 for fixed tao experiment OR 2 for fixed variance experiment" << endl;
	getline (cin, input);
	answer = lexical_cast<double>(input);
	if (answer == 1)
	{
		run_fixed_tao();
	}
	else if (answer == 2)
	{
		run_fixed_variance();
	}
	return 0;
}

void run_fixed_tao()
{
	string input;
	double tao = 0.0;
	string folder_to_test = "";
	string output_location = "";
	string thresholds_folder = "";
	string given_dist = "";

	cout << "Probability Threshold: " << endl;
	getline (cin, input);
	tao = lexical_cast<double>(input);

	cout << "Path to files to test (Full Path to the folder /): " << endl;
	getline (cin, input);
	folder_to_test = input;

	//get the location of the thresholds' file
	cout << "Where are the thresholds located? (Full Path to the folder)" << endl;
	getline (cin, input);
	thresholds_folder = input.c_str();

	//specify the location of output files
	cout << "Where do you want to put the output files? (Full Path to the folder /)" << endl;
	getline (cin, input);
	output_location = input;

	cout << "Specify the distribution" << endl;
	getline (cin, input);
	given_dist = input;
//	tao = 0.95;
//	folder_to_test = "/Users/besa/Thesis/temp/";
//	output_location = "/Users/besa/Thesis/ResultsMuenchen/";
//	thresholds_folder = "/Users/besa/Thesis/EuclideanShort_6/";


	//PBRQ
	int noSamples = NOSAMPLES;
	vector<string> variances;
	variances.push_back("0.2");
	variances.push_back("0.4");
	variances.push_back("0.6");
	variances.push_back("0.8");
	variances.push_back("1.0");
	variances.push_back("1.2");
	variances.push_back("1.4");
	variances.push_back("1.6");
	variances.push_back("1.8");
	variances.push_back("2.0");
	vector<string> distributions;
	distributions.push_back(given_dist);
//	distributions.push_back("uniform");
//	distributions.push_back("normal");
//	distributions.push_back("exponential");
	vector<string> folders;
	folders.push_back("Gun_Point_t60x6");
//	folders.push_back("synthetic_control_t60x6");
	Benchmark benchmark("MAIN LEVEL");
	benchmark.start(true);
	for(unsigned int f = 0; f < folders.size(); f++)
	{
		string base_output_filename = output_location + folders[f] + "_mnch_";
		for(unsigned int d = 0; d < distributions.size(); d++)
		{
			string output_filename = base_output_filename + distributions[d] + ".dat";
			ofstream pr_file;
			pr_file.open (output_filename.c_str());
                        for (unsigned int v = 0; v < variances.size(); v++)
			{
				string file_to_test = folder_to_test + folders[f] + "/" + folders[f] +
						"_mnch_" + distributions[d] + "_"+
						variances[v] + "_.dat";
				DataReader reader(file_to_test, COLLECTION_SIZE, LENGTH);
				reader.read(noSamples);
				Approximator approximator;
				approximator.reader = reader;
				approximator.noSamples = noSamples;
                                unsigned int drawnSamples = 0;
				string drawnIndeces = "";
				vector<int> processed;
				uniform_int<> ur(0, COLLECTION_SIZE - 1);
				variate_generator<mt19937&,uniform_int<> > drawSample(gen2, ur);
				double precision = 0.0;
				double recall = 0.0;
				double f1 = 0.0;
				double seconds = 0.0;

                                vector<int> fiveRandomIds;
                                fiveRandomIds.push_back(56);
                                fiveRandomIds.push_back(12); // ids of michele. besmira has these ids -1, because ids start from 0 while for michele from 1.
                                fiveRandomIds.push_back(40);
                                fiveRandomIds.push_back(21);
                                fiveRandomIds.push_back(30);

                                while (drawnSamples < NO_QUERIES)
				{
                                        //int drawnIndex = drawSample();

                                    int drawnIndex = fiveRandomIds[drawnSamples]-1;
                                    cerr << "query id is " << drawnIndex << endl;

					int exists = std::find(processed.begin(), processed.end(), drawnIndex)!=processed.end();
					if (exists == 1)
					{
						continue;
					}
					Benchmark benchmark_single(distributions[d] + " " + folders[f] + " " + variances[v] + " " + lexical_cast<std::string> (drawnSamples));
					benchmark_single.start(true);
					processed.push_back(drawnIndex);
					double eps_threshold = calculate_threshold(TOPK, drawnIndex, thresholds_folder, folders[f]);
					vector< vector<int> > m_euclidean = read_ground_truth(thresholds_folder, folders[f], eps_threshold);
					QueryEngine engine (eps_threshold * eps_threshold, tao, drawnIndex);
					engine.euclidean_matrix = m_euclidean;
					engine.approximator = approximator;
					string query_name = "timeseries" + boost::lexical_cast<std::string>(drawnIndex);
					engine.approximator.reader.query = engine.approximator.reader.series.find(query_name)->second;
					engine.approximator.epsilon = engine.epsilon;
					engine.approximator.tao = engine.tao;
					QueryEngine::PrTimeResults res = engine.queryPBRQ();
					precision += res.Precision;
					recall += res.Recall;
					f1 += res.F1;
					seconds += res.Time;
					drawnSamples++;
					benchmark_single.end(true);
				}
				precision = precision / NO_QUERIES;
				recall = recall / NO_QUERIES;
				f1 = f1 / NO_QUERIES;
				seconds = seconds / NO_QUERIES;
				pr_file << lexical_cast<std::string>(d) << " " << variances[v] << " " << precision << " " << recall << " " << f1 << " " << seconds << endl;
			}
			pr_file.close();
		}
	}
	benchmark.end(true);
}

void run_fixed_variance()
{
	string input;
	string fixed_var = "";
	string folder_to_test = "";
	string output_location = "";
	string thresholds_folder = "";
	string given_dist = "";

	cout << "Fixed Variance: " << endl;
	getline (cin, input);
	fixed_var = input;

	cout << "Path to files to test (Full Path to the folder /): " << endl;
	getline (cin, input);
	folder_to_test = input;

	//get the location of the thresholds' file
	cout << "Where are the thresholds located? (Full Path to the folder)" << endl;
	getline (cin, input);
	thresholds_folder = input.c_str();

	//specify the location of output files
	cout << "Where do you want to put the output files? (Full Path to the folder /)" << endl;
	getline (cin, input);
	output_location = input;

	cout << "Specify the distribution" << endl;
	getline (cin, input);
	given_dist = input;

//	fixed_var = "0.2";
//	folder_to_test = "/Users/besa/Thesis/temp/";
//	output_location = "/Users/besa/Thesis/ResultsMuenchen/fixed_var/";
//	thresholds_folder = "/Users/besa/Thesis/EuclideanShort_4/";


	//PBRQ
	int noSamples = NOSAMPLES;
	vector<string> tao;
	tao.push_back("0.05");
	tao.push_back("0.15");
	tao.push_back("0.25");
	tao.push_back("0.35");
	tao.push_back("0.45");
	tao.push_back("0.55");
	tao.push_back("0.65");
	tao.push_back("0.75");
	tao.push_back("0.85");
	tao.push_back("0.95");
	vector<string> distributions;
	distributions.push_back(given_dist);
//	distributions.push_back("uniform");
//	distributions.push_back("normal");
//	distributions.push_back("exponential");
	vector<string> folders;
	folders.push_back("Gun_Point_t60x6");
//	folders.push_back("synthetic_control_t60x6");
	Benchmark benchmark("MAIN LEVEL");
	benchmark.start(true);
	for(unsigned int f = 0; f < folders.size(); f++)
	{
		string base_output_filename = output_location + folders[f] + "_mnch_";
		for(unsigned int d = 0; d < distributions.size(); d++)
		{
			string output_filename = base_output_filename + distributions[d] + "_fixed_var.dat";
			ofstream pr_file;
			pr_file.open (output_filename.c_str());
                        for (unsigned int v = 0; v < tao.size(); v++)
			{
				string file_to_test = folder_to_test + folders[f] + "/" + folders[f] +
						"_mnch_" + distributions[d] + "_"+
						fixed_var + "_.dat";
				DataReader reader(file_to_test, COLLECTION_SIZE, LENGTH);
				reader.read(noSamples);
				Approximator approximator;
				approximator.reader = reader;
				approximator.noSamples = noSamples;
                                unsigned int drawnSamples = 0;
				string drawnIndeces = "";
				vector<int> processed;
				uniform_int<> ur(0, COLLECTION_SIZE - 1);
				variate_generator<mt19937&,uniform_int<> > drawSample(gen2, ur);
				double precision = 0.0;
				double recall = 0.0;
				double f1 = 0.0;
				double seconds = 0.0;
                                while (drawnSamples < NO_QUERIES)
				{
					int drawnIndex = drawSample();
					int exists = std::find(processed.begin(), processed.end(), drawnIndex)!=processed.end();
					if (exists == 1)
					{
						continue;
					}
					Benchmark benchmark_single(distributions[d] + " " + folders[f] + " " + fixed_var + " " + lexical_cast<std::string> (drawnSamples));
					benchmark_single.start(true);
					processed.push_back(drawnIndex);
					if (drawnIndex == 7)
						cout << "here";
					double eps_threshold = calculate_threshold(TOPK, drawnIndex, thresholds_folder, folders[f]);
					vector< vector<int> > m_euclidean = read_ground_truth(thresholds_folder, folders[f], eps_threshold);
					QueryEngine engine (eps_threshold * eps_threshold, lexical_cast<double> (tao[v]), drawnIndex);
					engine.euclidean_matrix = m_euclidean;
					engine.approximator = approximator;
					string query_name = "timeseries" + boost::lexical_cast<std::string>(drawnIndex);
					engine.approximator.reader.query = engine.approximator.reader.series.find(query_name)->second;
					engine.approximator.epsilon = engine.epsilon;
					engine.approximator.tao = engine.tao;
					QueryEngine::PrTimeResults res = engine.queryPBRQ();
					precision += res.Precision;
					recall += res.Recall;
					f1 += res.F1;
					seconds += res.Time;
					drawnSamples++;
					benchmark_single.end(true);
				}
				precision = precision / NO_QUERIES;
				recall = recall / NO_QUERIES;
				f1 = f1 / NO_QUERIES;
				seconds = seconds / NO_QUERIES;
				pr_file << lexical_cast<std::string>(d) << " " << tao[v] << " " << precision << " " << recall << " " << f1 << " " << seconds << endl;
			}
			pr_file.close();
		}
	}
	benchmark.end(true);
}

vector< vector<int> >  read_ground_truth(string folder, string dataset, double eps_threshold)
{
	string filename = folder + "euclidean_" + dataset + ".dat";
	ifstream seriesfile (filename.c_str());
	string line;
        unsigned int read = 0;
	vector< vector<int> > euclidean_matrix;
	if (seriesfile.is_open())
	{
		while ( seriesfile.good() )
		{
			  getline (seriesfile,line);
			  if (line.length() == 0)
				  continue;
			  vector<int> results_ts;
			  vector<string> data;
			  boost::split(data, line, boost::is_any_of(" "));
			  for(unsigned int k = 0; k < COLLECTION_SIZE; k++)
			  {
				  try
				  {
					  double distance = lexical_cast<double>(data[k]);
					  if (distance <= eps_threshold)
					  {
						  results_ts.push_back(k);
					  }
				  }
				  catch( boost::bad_lexical_cast const& )
				  {
					  cout << data[k] << "casting data to double error" << endl;
				  }
			  }
			  euclidean_matrix.push_back(results_ts);
                          // XXX read never changes?
			  if (read >= COLLECTION_SIZE)
				  break;
		}
	}
	seriesfile.close();
	return euclidean_matrix;
}

double calculate_threshold(int topk, int query_i, string folder, string dataset)
{
	string filename = folder + "euclidean_" + dataset + ".dat";
	ifstream seriesfile (filename.c_str());
	string line;
	vector<double> distances;
	if (seriesfile.is_open())
	{
		int i = 0;
		while ( seriesfile.good() )
		{
			getline (seriesfile,line);
			if (i == query_i)
			{
				vector<string> data;
				boost::split(data, line, boost::is_any_of(" "));
				for(unsigned int k = 0; k < COLLECTION_SIZE; k++)
				{
					try
					{
						double distance = lexical_cast<double>(data[k]);
						distances.push_back(distance);
 					}
					catch( boost::bad_lexical_cast const& )
					{
						cout << data[k] << "casting data to double error" << endl;
					}
				}
				break;
			}
			i++;
		}
	}
	seriesfile.close();
	std::sort(distances.begin(), distances.end());
	return distances[topk - 1];
}

