/*
 * QueryEngine.cpp
 *
 *  Created on: May 9, 2011
 *      Author: besa
 */

#include "timeseriescollection.h"
#include <math.h>
#include <boost/math/special_functions/erf.hpp>
#include <fstream>
#include <iostream>
#include <string>
#include <stdio.h>
#include <dirent.h>
#include<sys/stat.h>
#include<sys/types.h>

#include "QueryEngine.h"
#include "Benchmark.h"

using namespace boost;
using namespace std;

QueryEngine::QueryEngine() {

}

QueryEngine::QueryEngine(TimeSeriesCollection c, long int s,
                         long int e, double tao_value, double r_value, int i) {
    this->allSeries = c;
    this->ts = s;
    this->te = e;
    this->tao = tao_value;
    this->r = r_value;
    this->index_query = i;
}

QueryEngine::~QueryEngine() {
    // TODO Auto-generated destructor stub
}

time_duration QueryEngine::query()
{
    //resultset.clear(); // xxx added by michele. seems to be needed! resultset is not always empty.
    Benchmark benchmark("SINGLE QUERY");
    benchmark.start(false);
    vector<double> E;
    vector<double> Var;
    vector<double> C_Sum;
    for(unsigned long int j = 0; j < this->allSeries.sequences.size(); j++)
    {
        E.push_back(0);
        Var.push_back(0);
        C_Sum.push_back(0);
        resultset.push_back(1);
    }
    //calculate r-limit
    double r_limit = sqrt(2) * boost::math::erf_inv(2 * this->tao - 1);
    long int length = te - ts + 1;
    double square_r = pow(this->r, 2);
    //observation is considered as the mean
    for(long int j = ts; j < te; j++)
    {
        long int tk = j;
        //update E and Dst for element in the timeseries collection
        for (unsigned long int i = 0; i < this->allSeries.sequences.size(); i++)
        {
            if (this->resultset[i] == 0)
                continue;
            double diff_mu = pow(this->allSeries.sequences[i].sequence[tk].observation -
                                 this->allSeries.sequences[this->index_query].sequence[tk].observation, 2);
            E[i] += diff_mu;
            //			double sum_sigma = pow(this->allSeries.sequences[i].sequence[tk].sigma, 2) +
            //					pow(this->allSeries.sequences[this->index_query].sequence[tk].sigma, 2);
            double sum_sigma = this->allSeries.sequences[i].sequence[tk].variance +
                               this->allSeries.sequences[this->index_query].sequence[tk].variance;
            Var[i] += 4 * sum_sigma * diff_mu;
            //equation 27 in the paper
            double current_sum = C_Sum[i] + diff_mu;
            double condition_value = square_r - sum_sigma * length +
                                     sqrt(C_Sum[i] * current_sum);
            C_Sum[i] = current_sum;
            if (condition_value >= 0)
            {
                //equation 25 in the paper
                double r_norm = (square_r - E[i]) / sqrt(Var[i]);
                if (r_norm < r_limit)
                {
                    this->resultset[i] = 0;
                }
            }
            else if (j == te)
            {
                //equation 25 in the paper
                double r_norm = (square_r - E[i]) / sqrt(Var[i]);
                if (r_norm < r_limit)
                {
                    this->resultset[i] = 0;
                }
            }

        }
    }
    benchmark.end(false);

    if(this->ROOT_FOLDER.compare("") != 0) {
	this->write_to_file();
    }
    return benchmark.getDuration();
}

void QueryEngine::write_to_file()
{
    ofstream result_file;
    string filename = this->ROOT_FOLDER  + boost::lexical_cast<std::string>(this->index_query) + ".dat";
    //	rmdir(this->ROOT_FOLDER.c_str());
    //	mkdir(this->ROOT_FOLDER.c_str(), 0777);
    result_file.open (filename.c_str());
    for (unsigned int i = 0; i < this->resultset.size(); i++)
    {
        if (this->resultset[i] == 1)
        {
            result_file << i << endl;
        }
    }
    result_file.close();
}
